<?php

namespace Acts\CamdramBundle\Entity;

use Doctrine\ORM\EntityRepository;

/**
 * WeekRepository
 *
 * This class was generated by the Doctrine ORM. Add your own custom
 * repository methods below.
 * @extends EntityRepository<WeekName>
 */
class WeekNameRepository extends EntityRepository
{
    public function findAt(\DateTime $date)
    {
        $qb = $this->createQueryBuilder('p');
        $query = $qb->where('p.start_at = :now')
            ->setParameter('now', $date)
            ->getQuery();

        return $query->getOneOrNullResult();
    }

    public function findAfter($date, $limit)
    {
        $qb = $this->createQueryBuilder('p');
        $query = $qb->where('p.start_at >= :date')
            ->setParameter('date', $date)
            ->orderBy('p.start_at', 'ASC')
            ->setMaxResults($limit)
            ->getQuery();

        return $query->getResult();
    }

    public function findBetween(\DateTime $start_date, \DateTime $end_date, $limit = null)
    {
        $qb = $this->createQueryBuilder('p');
        $qb->where('p.start_at >= :start_date')
            ->andWhere('p.start_at < :end_date')
            ->setParameter('start_date', $start_date)
            ->setParameter('end_date', $end_date)
            ->orderBy('p.start_at', 'ASC');
        if ($limit > 0) {
            $qb->setMaxResults($limit);
        }

        return $qb->getQuery()->getResult();
    }

    public function findIntersecting(\DateTime $start, \DateTime $end)
    {
        $qb = $this->createQueryBuilder('p');
        $query = $qb
            ->where($qb->expr()->andX('p.start_at < :start', 'p.end_at >= :start'))
            ->orWhere($qb->expr()->andX('p.start_at < :end', 'p.end_at >= :end'))
            ->orWhere($qb->expr()->andX('p.start_at > :start', 'p.end_at <= :end'))
            ->setParameter('start', $start)->setParameter('end', $end)
            ->getQuery();

        return $query->getResult();
    }

    public function getByYearPeriodAndSlug($year, $period, $slug)
    {
        $year = (int) $year;
        $start = new \DateTime($year.'-01-01');
        $end = new \DateTime(($year + 1).'-01-01');

        $query = $this->createQueryBuilder('w')
            ->join('w.time_period', 'p')
            ->where('w.slug = :slug')
            ->andWhere('w.start_at >= :start_date')
            ->andWhere('w.start_at < :end_date')
            ->andWhere('p.slug = :period')
            ->setParameter('slug', $slug)
            ->setParameter('period', $period)
            ->setParameter('start_date', $start)
            ->setParameter('end_date', $end)
            ->setMaxResults(1)
            ->getQuery();

        return $query->getOneOrNullResult();
    }
}
